import { ForceLink, Simulation, SimulationLinkDatum } from 'd3-force';
import { IPosition } from '../../common';
import { ISimulationNode, ISimulationEdge } from '../shared';
import { Emitter } from '../../utils/emitter.utils';
export declare enum D3SimulatorEngineEventType {
    TICK = "tick",
    END = "end",
    SIMULATION_START = "simulation-start",
    SIMULATION_PROGRESS = "simulation-progress",
    SIMULATION_END = "simulation-end",
    NODE_DRAG = "node-drag",
    SETTINGS_UPDATE = "settings-update"
}
export interface ID3SimulatorEngineSettingsAlpha {
    alpha: number;
    alphaMin: number;
    alphaDecay: number;
    alphaTarget: number;
}
export interface ID3SimulatorEngineSettingsCentering {
    x: number;
    y: number;
    strength: number;
}
export interface ID3SimulatorEngineSettingsCollision {
    radius: number;
    strength: number;
    iterations: number;
}
export interface ID3SimulatorEngineSettingsLinks {
    distance: number;
    strength?: number;
    iterations: number;
}
export interface ID3SimulatorEngineSettingsManyBody {
    strength: number;
    theta: number;
    distanceMin: number;
    distanceMax: number;
}
export interface ID3SimulatorEngineSettingsPositioning {
    forceX: {
        x: number;
        strength: number;
    };
    forceY: {
        y: number;
        strength: number;
    };
}
export interface ID3SimulatorEngineSettings {
    isPhysicsEnabled: boolean;
    alpha: ID3SimulatorEngineSettingsAlpha;
    centering: ID3SimulatorEngineSettingsCentering | null;
    collision: ID3SimulatorEngineSettingsCollision | null;
    links: ID3SimulatorEngineSettingsLinks;
    manyBody: ID3SimulatorEngineSettingsManyBody | null;
    positioning: ID3SimulatorEngineSettingsPositioning | null;
}
export declare type ID3SimulatorEngineSettingsUpdate = Partial<ID3SimulatorEngineSettings>;
export declare const getManyBodyMaxDistance: (linkDistance: number) => number;
export declare const DEFAULT_SETTINGS: ID3SimulatorEngineSettings;
export interface ID3SimulatorProgress {
    progress: number;
}
export interface ID3SimulatorGraph {
    nodes: ISimulationNode[];
    edges: ISimulationEdge[];
}
export interface ID3SimulatorNodeId {
    id: number;
}
export interface ID3SimulatorSettings {
    settings: ID3SimulatorEngineSettings;
}
interface IRunSimulationOptions {
    isUpdatingSettings: boolean;
}
export declare type D3SimulatorEvents = {
    [D3SimulatorEngineEventType.TICK]: ID3SimulatorGraph;
    [D3SimulatorEngineEventType.END]: ID3SimulatorGraph;
    [D3SimulatorEngineEventType.SIMULATION_START]: undefined;
    [D3SimulatorEngineEventType.SIMULATION_PROGRESS]: ID3SimulatorGraph & ID3SimulatorProgress;
    [D3SimulatorEngineEventType.SIMULATION_END]: ID3SimulatorGraph;
    [D3SimulatorEngineEventType.NODE_DRAG]: ID3SimulatorGraph;
    [D3SimulatorEngineEventType.SETTINGS_UPDATE]: ID3SimulatorSettings;
};
export declare class D3SimulatorEngine extends Emitter<D3SimulatorEvents> {
    protected readonly linkForce: ForceLink<ISimulationNode, SimulationLinkDatum<ISimulationNode>>;
    protected readonly simulation: Simulation<ISimulationNode, undefined>;
    protected readonly settings: ID3SimulatorEngineSettings;
    protected _edges: ISimulationEdge[];
    protected _nodes: ISimulationNode[];
    protected _nodeIndexByNodeId: Record<number, number>;
    protected _isDragging: boolean;
    protected _isStabilizing: boolean;
    constructor(settings?: ID3SimulatorEngineSettings);
    getSettings(): ID3SimulatorEngineSettings;
    setSettings(settings: ID3SimulatorEngineSettingsUpdate): void;
    startDragNode(): void;
    dragNode(data: ID3SimulatorNodeId & IPosition): void;
    endDragNode(data: ID3SimulatorNodeId): void;
    activateSimulation(): void;
    private fixDefinedNodes;
    addData(data: ID3SimulatorGraph): void;
    clearData(): void;
    setData(data: ID3SimulatorGraph): void;
    updateData(data: ID3SimulatorGraph): void;
    simulate(): void;
    startSimulation(data: ID3SimulatorGraph): void;
    updateSimulation(data: ID3SimulatorGraph): void;
    stopSimulation(): void;
    protected initSimulation(settings: ID3SimulatorEngineSettingsUpdate): void;
    protected runSimulation(options?: IRunSimulationOptions): void;
    protected setNodeIndexByNodeId(): void;
    fixNodes(nodes?: ISimulationNode[]): void;
    releaseNodes(nodes?: ISimulationNode[]): void;
}
export {};
